-- ======================================================================
-- == VOICE RECOGNITION & GEMINI CORRECTION SCRIPT (v25 - No Start Delay)==
-- ======================================================================
-- == Base: User's v22 script with strict prompt (v24).                ==
-- == Modification: Removed the initial delay before starting voice    ==
-- ==               recognition on auto-start or button press.         ==
-- == Goal: Immediate Start + Strictly Conditional Translation + Very  ==
-- ==       Minimal Correction.                                        ==
-- ======================================================================

-- == Step 0: Initial Setup and Bindings ==
local scriptStartTime = os.time()

this = activity or service
if not this then return false end

local function my_speak_service(message) -- Using function name from user's working base
  pcall(function()
    if this and this.service and this.service.asyncSpeak then
      this.service.asyncSpeak(tostring(message or "Walang mensahe"))
    elseif this and this.asyncSpeak then
       this.asyncSpeak(tostring(message or "Walang mensahe"))
    end
  end)
end

if not ((this and this.service and this.service.asyncSpeak) or (this and this.asyncSpeak)) then return false end

-- Bind necessary classes
local Http, SpeechRecognizer, RecognizerIntent, Intent, Handler, ComponentName, Uri, InputMethodManager, View
local ClipboardManager, ClipData, Context

local bindSuccess, bindResult = pcall(function()
  Http = luajava.bindClass("com.androlua.Http")
  SpeechRecognizer = luajava.bindClass("android.speech.SpeechRecognizer")
  RecognizerIntent = luajava.bindClass("android.speech.RecognizerIntent")
  Intent = luajava.bindClass("android.content.Intent")
  Handler = luajava.bindClass("android.os.Handler")
  ComponentName = luajava.bindClass("android.content.ComponentName")
  Uri = luajava.bindClass("android.net.Uri")
  InputMethodManager = luajava.bindClass("android.view.inputmethod.InputMethodManager")
  View = luajava.bindClass("android.view.View")
  ClipboardManager = luajava.bindClass("android.content.ClipboardManager")
  ClipData = luajava.bindClass("android.content.ClipData")
  Context = luajava.bindClass("android.content.Context")
  cjson = require "cjson"
end)

if not bindSuccess then
  my_speak_service("Fatal Error: Hindi ma-bind ang mga klase.")
  return false
end

-- Global variables
local currentRecognizer = nil
local isListening = false
local geminiApiKey = os.getenv("GEMINI_API_KEY") or "AIzaSyArz06zWTwsG6LVd9yD0fZ0gCvcHOm9C9k" -- THIS LINE IS UNCHANGED AS PER INSTRUCTION

-- === GEMINI API FUNCTION === (STRICT ENGLISH PROMPT)
function correctGrammarWithGemini(rawText)
  -- Note: API Key check logic remains as provided in the user's script
  if not geminiApiKey or geminiApiKey == "ILAGAY_ANG_IYONG_API_KEY_DITO" then my_speak_service("Error: Gemini API Key is missing..."); return end
  if not rawText or rawText == "" then return end

  local modelName = "gemini-1.5-flash-latest"
  local geminiUrl = "https://generativelanguage.googleapis.com/v1beta/models/" .. modelName .. ":generateContent?key=" .. geminiApiKey

  -- *** STRONGLY REVISED ENGLISH PROMPT FOR STRICT BEHAVIOR (ONLY THIS SECTION REMAINS MODIFIED) ***
  local correctionPrompt = string.format([[
**STRICT INSTRUCTIONS:**

1.  **CHECK FOR TRANSLATION COMMAND:** Does the input text **BEGIN EXACTLY** with "translate to english" or "isalin sa english" (case-insensitive)? **This is the ONLY trigger for translation.**

2.  **MODE 1: TRANSLATE (ONLY IF COMMAND IS PRESENT):**
    * If the input **BEGINS** with the translation command:
        * **Ignore** the command phrase ("translate to english" or "isalin sa english") in your output.
        * Translate **ONLY the rest of the input text** (after the command) into **PERFECT, STANDARD ENGLISH**.
        * Ensure the grammar, punctuation, and flow of the resulting English translation are correct.
        * Return **ONLY** the final, polished, **complete English translation**. **DO NOT** include any extra text, explanations, or the original phrase.

3.  **MODE 2: MINIMAL CORRECTION (IF COMMAND IS ABSENT):**
    * If the input **DOES NOT BEGIN** with the translation command:
        * You **MUST NOT** translate the text.
        * You **MUST NOT** change the language of the text (it is likely Taglish or Tagalog).
        * Perform **ABSOLUTELY MINIMAL CORRECTION** on the **ENTIRE ORIGINAL INPUT TEXT**.
        * **ALLOWED CORRECTIONS:** Fix only obvious spelling mistakes, very basic punctuation (add/remove periods, commas, question marks, exclamation points where clearly needed), and capitalize the first letter of sentences.
        * **STRICTLY FORBIDDEN CORRECTIONS:** You **MUST NOT** change any words, add any words, remove any words. You **MUST NOT** alter the original meaning or tone. The **WORD COUNT OF THE OUTPUT MUST BE IDENTICAL TO THE WORD COUNT OF THE INPUT.** Maintain the mix of languages (Taglish) exactly as provided.
        * Return **ONLY** the minimally corrected **ORIGINAL INPUT TEXT**. **DO NOT** include any extra text, explanations, or translations.

**Input Text:**
"%s"
]], tostring(rawText)) -- Ensure rawText is string
  -- *** END OF MODIFIED SECTION (correctionPrompt) ***

  local requestBodyTable = { contents = { { parts = { { text = correctionPrompt } } } }, generationConfig = { temperature = 0.1, maxOutputTokens = 1000 } } -- Note: Original structure used here
  local successEncode, jsonResult = pcall(cjson.encode, requestBodyTable)
  if not successEncode then my_speak_service("Error encoding request."); return end
  local jsonRequestBodyString = jsonResult
  local headers = { ["Content-Type"] = "application/json" }

  local successHttpPostCall, errorHttpPostCall = pcall(function()
    Http.post(geminiUrl, jsonRequestBodyString, headers, function(status, responseBody)
      if status == 200 then
        local responseData = nil
        local successDecode, decodeResult = pcall(cjson.decode, responseBody or "")
        if successDecode and decodeResult then
          responseData = decodeResult
          local extractedText = nil
          local pcallSuccessExtract, pcallResultExtract = pcall(function()
             if responseData.candidates and responseData.candidates[1] and responseData.candidates[1].content and responseData.candidates[1].content.parts and responseData.candidates[1].content.parts[1] and responseData.candidates[1].content.parts[1].text then extractedText = responseData.candidates[1].content.parts[1].text
             elseif responseData.candidates and responseData.candidates[1] and responseData.candidates[1].finishReason == "SAFETY" then extractedText = "Gemini blocked: Safety."; my_speak_service(extractedText)
             elseif responseData.promptFeedback and responseData.promptFeedback.blockReason then extractedText = "Gemini blocked: Prompt issue."; my_speak_service(extractedText)
             else extractedText = nil; my_speak_service("Hindi maintindihan ang response structure.") end
          end)

          if pcallSuccessExtract and extractedText and extractedText ~= "Gemini blocked: Safety." and extractedText ~= "Gemini blocked: Prompt issue." then
             local cleanedText = extractedText:gsub("^[%s\n]*(.-)%s*$", "%1")
             my_speak_service("Resulta: " .. cleanedText)

             -- *** Text Insertion Logic EXACTLY from User's Working Example ***
             local successAppend = pcall(function()
               local editText = this.service.getEditText()
               if editText then
                  local currentText = ""; local successGetText, textVal = pcall(function() return editText.getText().toString() end); if successGetText and textVal then currentText = textVal end
                  local newText = nil
                  if currentText == "" then newText = cleanedText else
                      if string.sub(currentText, -1) == " " then newText = currentText .. cleanedText
                      else newText = currentText .. " " .. cleanedText end
                  end
                  -- Note: Original setText call is kept as in the user's provided script
                  local successSetText, setError = pcall(this.service.setText, editText, newText)
                  if successSetText then
                     pcall(function() editText.setSelection(string.len(newText)) end)
                  else
                      my_speak_service("Error updating text field. Nasa clipboard na lang.")
                      pcall(function() if this and Context and ClipboardManager and ClipData then local clipboardMgr = this.getSystemService(Context.CLIPBOARD_SERVICE); if clipboardMgr then local clipdata = ClipData.newPlainText("corrected_text", cleanedText); clipboardMgr.setPrimaryClip(clipdata); end end end)
                  end
               else
                  my_speak_service("Error: Cannot find focused text field. Nasa clipboard na lang.")
                   -- Fallback to clipboard if getEditText() returned nil
                  pcall(function() if this and Context and ClipboardManager and ClipData then local clipboardMgr = this.getSystemService(Context.CLIPBOARD_SERVICE); if clipboardMgr then local clipdata = ClipData.newPlainText("corrected_text", cleanedText); clipboardMgr.setPrimaryClip(clipdata); end end end)
               end
             end)
             if not successAppend then my_speak_service("Error occurred during text appending logic.") end
             -- *** END of Text Insertion Logic from User's Working Example ***
          elseif not extractedText then my_speak_service("Error extracting text from Gemini response.") end
        else my_speak_service("Error decoding Gemini JSON response.") end
      else
        my_speak_service("Gemini API Error. Status: " .. (status or "unknown"))
      end
    end)
  end)
  if not successHttpPostCall then
     my_speak_service("Critical Error: Failed to initiate Http post call.")
  end
end

-- === SPEECH RECOGNITION FUNCTIONS === (Kept as in User's Provided Script)
function stopListening()
  if isListening and currentRecognizer then
    isListening = false
    local stopSuccess, stopResult = pcall(function()
      currentRecognizer.stopListening()
      currentRecognizer.cancel()
      currentRecognizer.destroy()
    end)
    currentRecognizer = nil
    if stopSuccess then
      my_speak_service("Huminto sa pakikinig.")
    else
      my_speak_service("Nagka-error sa paghinto.")
    end
  end
end

function startVoiceGrammarCorrection()
    if isListening then
      my_speak_service("Nakikinig na.")
      return
    end

    isListening = true
    local speechNotificationSent = false
    local recognizer = nil
    local intent = nil
    local errorOccurred = false

    -- Attempt Specific Engine First (Kept as in User's Provided Script)
    local targetPackage = "com.google.android.tts"
    local targetClass = "com.google.android.apps.speech.tts.googletts.service.GoogleTTSRecognitionService"
    local compName = nil

    local createCompSuccess, createCompResult = pcall(function() if ComponentName then compName = ComponentName(targetPackage, targetClass) else error("ComponentName not bound") end end)

    if createCompSuccess and compName then
       local createSpecificSuccess, createSpecificResult = pcall(function() if this and SpeechRecognizer then recognizer = SpeechRecognizer.createSpeechRecognizer(this, compName) else error("'this' or SpeechRecognizer not available") end end);
       if not createSpecificSuccess or recognizer == nil then recognizer = nil end
    end

    -- Fallback to Default Recognizer (Kept as in User's Provided Script)
    if recognizer == nil then
       local createDefaultSuccess, createDefaultResult = pcall(function() if this and SpeechRecognizer then recognizer = SpeechRecognizer.createSpeechRecognizer(this) else error("'this' or SpeechRecognizer not available") end; if recognizer == nil then error("createSpeechRecognizer(default) returned nil") end end);
       if not createDefaultSuccess or recognizer == nil then my_speak_service("Fatal Error: Hindi makagawa ng recognizer (default)."); errorOccurred = true end
    end

    if errorOccurred or recognizer == nil then my_speak_service("Huminto dahil sa error sa recognizer setup."); isListening = false; return end
    currentRecognizer = recognizer

    -- Setup Intent (Kept as in User's Provided Script)
    local setupIntentSuccess, setupIntentResult = pcall(function()
        if Intent and RecognizerIntent then
           intent = Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH)
           intent.putExtra(RecognizerIntent.EXTRA_LANGUAGE_MODEL, RecognizerIntent.LANGUAGE_MODEL_FREE_FORM)
           intent.putExtra(RecognizerIntent.EXTRA_PROMPT, "Magsalita ka...")
           intent.putExtra(RecognizerIntent.EXTRA_LANGUAGE, "fil-PH")
           intent.putExtra(RecognizerIntent.EXTRA_SPEECH_INPUT_MINIMUM_LENGTH_MILLIS, 500)
           intent.putExtra(RecognizerIntent.EXTRA_SPEECH_INPUT_COMPLETE_SILENCE_LENGTH_MILLIS, 1000)
           intent.putExtra(RecognizerIntent.EXTRA_PARTIAL_RESULTS, false)
           intent.putExtra(RecognizerIntent.EXTRA_MAX_RESULTS, 1)
        else error("Intent or RecognizerIntent not bound") end; if intent == nil then error("Intent creation failed") end
    end)

    if not setupIntentSuccess or intent == nil then my_speak_service("Error sa pag-setup ng Recognition Intent."); isListening = false; pcall(function() if recognizer then recognizer.destroy() end end); currentRecognizer = nil; return end

    -- Listener setup (Kept as in User's Provided Script)
    local listener = nil;
    local listenerSetupSuccess, listenerSetupResult = pcall(function()
      listener = luajava.createProxy("android.speech.RecognitionListener", {
        onReadyForSpeech = function(params) if not speechNotificationSent then my_speak_service("Listening..."); speechNotificationSent = true end end,
        onBeginningOfSpeech = function() end, onRmsChanged = function(rmsdB) end, onBufferReceived = function(buffer) end, onEndOfSpeech = function() end,
        onResults = function(results)
            isListening = false; local currentRec = currentRecognizer; currentRecognizer = nil; local recognizedText = "Error processing results"
            local pcallSuccess, pcallResult = pcall(function()
               if results == nil then error("Results object is nil") end;
               local matches = results.getStringArrayList(SpeechRecognizer.RESULTS_RECOGNITION);
               if matches ~= nil and matches.size() > 0 then recognizedText = matches.get(0); if recognizedText == nil or recognizedText == "" then recognizedText = "Walang nakuha na resulta" end
               else recognizedText = "Walang nakuhang tugma" end
            end);
            if pcallSuccess then
               if recognizedText ~= "Walang nakuhang tugma" and recognizedText ~= "Error processing results" and recognizedText ~= "Walang nakuha na resulta" then
                  local handler = Handler(); handler.postDelayed(function() correctGrammarWithGemini(recognizedText) end, 200)
               else my_speak_service("Hindi nakuha ang sinabi. Pakiulit.") end
            else my_speak_service("Listener: Error sa pagkuha ng resulta. Pakiulit."); end;
            if currentRec then pcall(function() currentRec.destroy() end) end
        end,
        onError = function(error) isListening = false; local currentRec = currentRecognizer; currentRecognizer = nil; local errorMsg = "Voice recognition error."; local errorCode = tonumber(error);
           if errorCode == SpeechRecognizer.ERROR_NO_MATCH then errorMsg = "Walang narinig. Pakiulit." elseif errorCode == SpeechRecognizer.ERROR_SPEECH_TIMEOUT then errorMsg = "Walang nagsalita. Pakiulit." elseif errorCode == SpeechRecognizer.ERROR_NETWORK or errorCode == SpeechRecognizer.ERROR_NETWORK_TIMEOUT then errorMsg = "Network error. Check connection." elseif errorCode == SpeechRecognizer.ERROR_INSUFFICIENT_PERMISSIONS then errorMsg = "Kailangan ng mic permission." else errorMsg = "Recognition error code: "..tostring(error) end;
           my_speak_service(errorMsg);
           if currentRec then pcall(function() currentRec.destroy() end) end
        end,
        onPartialResults = function(partialResults) end, onEvent = function(eventType, params) end }); if recognizer and listener then recognizer.setRecognitionListener(listener) else error("Recognizer or Listener is nil before setting") end end)

    if not listenerSetupSuccess then my_speak_service("Error sa pag-setup ng listener."); isListening = false; pcall(function() if recognizer then recognizer.destroy() end end); currentRecognizer = nil; return end

    -- Use pcall directly around startListening (Kept as in User's Provided Script)
    local startSuccess, startResult = pcall(function() if recognizer and intent then recognizer.startListening(intent) else error("Recognizer or Intent is nil before starting") end end)
    if not startSuccess then my_speak_service("Fatal Error starting voice recognition."); isListening = false; pcall(function() if recognizer then recognizer.destroy() end end); currentRecognizer = nil; end
end
-- === END OF INJECTED SPEECH RECOGNITION FUNCTIONS ===


-- Button click event (Kept as in User's Provided Script)
function onButtonClick()
    startVoiceGrammarCorrection()
end

-- === Simulan ang Script === (REMOVED DELAY HERE)
local autoStart = true
if autoStart then
   -- handler.postDelayed(function() -- Removed delay start
      startVoiceGrammarCorrection() -- Start immediately
   -- end, 1000); -- Removed delay end and time
else
   my_speak_service("Handa na. Pindutin ang button.")
end

return true